/**
 * This file is part of emf2gv : an eclipse plugin that allows to
 * generate a graphical representation of an EMF model.
 *
 * Copyright 2010-2011 Jean-Francois Brazeau
 * 
 * emf2gv is free software: you can redistribute it and/or modify
 * it under the terms of either:
 * 
 *      a) the GNU Lesser General Public License as published by
 *       the Free Software Foundation, either version 3 of the License, or
 *       (at your option) any later version.
 *  or
 *      b) the Eclipse Public License version 1.0 as published by
 *       the Eclipse Foundation.
 * 
 * emf2gv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with emf2gv.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * You should have received a copy of the  Eclipse Public License
 * along with emf2gv.  If not, see <http://www.eclipse.org/legal/epl-v10.html>.
 */
package org.emftools.emf2gv.graphdesc.provider;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.edit.provider.ChangeNotifier;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IChangeNotifier;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.emftools.emf2gv.graphdesc.util.GraphdescAdapterFactory;

/**
 * This is the factory that is used to provide the interfaces needed to support Viewers.
 * The adapters generated by this factory convert EMF adapter notifications into calls to {@link #fireNotifyChanged fireNotifyChanged}.
 * The adapters also support Eclipse property sheets.
 * Note that most of the adapters are shared among multiple instances.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class GraphdescItemProviderAdapterFactory extends GraphdescAdapterFactory implements ComposeableAdapterFactory, IChangeNotifier, IDisposable {
	/**
	 * This keeps track of the root adapter factory that delegates to this adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ComposedAdapterFactory parentAdapterFactory;

	/**
	 * This is used to implement {@link org.eclipse.emf.edit.provider.IChangeNotifier}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected IChangeNotifier changeNotifier = new ChangeNotifier();

	/**
	 * This keeps track of all the supported types checked by {@link #isFactoryForType isFactoryForType}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected Collection<Object> supportedTypes = new ArrayList<Object>();

	/**
	 * This constructs an instance.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public GraphdescItemProviderAdapterFactory() {
		supportedTypes.add(IEditingDomainItemProvider.class);
		supportedTypes.add(IStructuredItemContentProvider.class);
		supportedTypes.add(ITreeItemContentProvider.class);
		supportedTypes.add(IItemLabelProvider.class);
		supportedTypes.add(IItemPropertySource.class);
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.ClassFigure} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ClassFigureItemProvider classFigureItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.ClassFigure}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createClassFigureAdapter() {
		if (classFigureItemProvider == null) {
			classFigureItemProvider = new ClassFigureItemProvider(this);
		}

		return classFigureItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.AttributeFigure} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected AttributeFigureItemProvider attributeFigureItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.AttributeFigure}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createAttributeFigureAdapter() {
		if (attributeFigureItemProvider == null) {
			attributeFigureItemProvider = new AttributeFigureItemProvider(this);
		}

		return attributeFigureItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.ReferenceFigure} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ReferenceFigureItemProvider referenceFigureItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.ReferenceFigure}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createReferenceFigureAdapter() {
		if (referenceFigureItemProvider == null) {
			referenceFigureItemProvider = new ReferenceFigureItemProvider(this);
		}

		return referenceFigureItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.RichAttributeFigure} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected RichAttributeFigureItemProvider richAttributeFigureItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.RichAttributeFigure}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createRichAttributeFigureAdapter() {
		if (richAttributeFigureItemProvider == null) {
			richAttributeFigureItemProvider = new RichAttributeFigureItemProvider(this);
		}

		return richAttributeFigureItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.RichReferenceFigure} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected RichReferenceFigureItemProvider richReferenceFigureItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.RichReferenceFigure}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createRichReferenceFigureAdapter() {
		if (richReferenceFigureItemProvider == null) {
			richReferenceFigureItemProvider = new RichReferenceFigureItemProvider(this);
		}

		return richReferenceFigureItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.DynamicPropertyOverrider} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DynamicPropertyOverriderItemProvider dynamicPropertyOverriderItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.DynamicPropertyOverrider}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createDynamicPropertyOverriderAdapter() {
		if (dynamicPropertyOverriderItemProvider == null) {
			dynamicPropertyOverriderItemProvider = new DynamicPropertyOverriderItemProvider(this);
		}

		return dynamicPropertyOverriderItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.Filter} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected FilterItemProvider filterItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.Filter}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createFilterAdapter() {
		if (filterItemProvider == null) {
			filterItemProvider = new FilterItemProvider(this);
		}

		return filterItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.emftools.emf2gv.graphdesc.GVFigureDescription} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected GVFigureDescriptionItemProvider gvFigureDescriptionItemProvider;

	/**
	 * This creates an adapter for a {@link org.emftools.emf2gv.graphdesc.GVFigureDescription}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createGVFigureDescriptionAdapter() {
		if (gvFigureDescriptionItemProvider == null) {
			gvFigureDescriptionItemProvider = new GVFigureDescriptionItemProvider(this);
		}

		return gvFigureDescriptionItemProvider;
	}

	/**
	 * This returns the root adapter factory that contains this factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ComposeableAdapterFactory getRootAdapterFactory() {
		return parentAdapterFactory == null ? this : parentAdapterFactory.getRootAdapterFactory();
	}

	/**
	 * This sets the composed adapter factory that contains this factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setParentAdapterFactory(ComposedAdapterFactory parentAdapterFactory) {
		this.parentAdapterFactory = parentAdapterFactory;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object type) {
		return supportedTypes.contains(type) || super.isFactoryForType(type);
	}

	/**
	 * This implementation substitutes the factory itself as the key for the adapter.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter adapt(Notifier notifier, Object type) {
		return super.adapt(notifier, this);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object adapt(Object object, Object type) {
		if (isFactoryForType(type)) {
			Object adapter = super.adapt(object, type);
			if (!(type instanceof Class<?>) || (((Class<?>)type).isInstance(adapter))) {
				return adapter;
			}
		}

		return null;
	}

	/**
	 * This adds a listener.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void addListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.addListener(notifyChangedListener);
	}

	/**
	 * This removes a listener.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void removeListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.removeListener(notifyChangedListener);
	}

	/**
	 * This delegates to {@link #changeNotifier} and to {@link #parentAdapterFactory}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void fireNotifyChanged(Notification notification) {
		changeNotifier.fireNotifyChanged(notification);

		if (parentAdapterFactory != null) {
			parentAdapterFactory.fireNotifyChanged(notification);
		}
	}

	/**
	 * This disposes all of the item providers created by this factory. 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void dispose() {
		if (gvFigureDescriptionItemProvider != null) gvFigureDescriptionItemProvider.dispose();
		if (classFigureItemProvider != null) classFigureItemProvider.dispose();
		if (attributeFigureItemProvider != null) attributeFigureItemProvider.dispose();
		if (richAttributeFigureItemProvider != null) richAttributeFigureItemProvider.dispose();
		if (referenceFigureItemProvider != null) referenceFigureItemProvider.dispose();
		if (richReferenceFigureItemProvider != null) richReferenceFigureItemProvider.dispose();
		if (dynamicPropertyOverriderItemProvider != null) dynamicPropertyOverriderItemProvider.dispose();
		if (filterItemProvider != null) filterItemProvider.dispose();
	}

}
